/*
 * Licensed Materials - Property of IBM
 * 5648-C63
 * (C) Copyright IBM Corp. 1999, 2000
 */

/* Includes */
#include <malloc.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>
#include "BipSampPluginUtil.h"

/**********************************************************************************************/
/* Plug-in node utility functions                                                             */
/*   CciCharIsSpace : Returns 0 if CciChar is not space, otherwise returns 1                  */
/*   CciCharIsAlnum : Returns 0 if CciChar is not alpha-numberic, otherwise returns 1         */
/*   CciCharCpy     : Copy CciChar strings, like strcpy                                       */
/*   CciCharNcpy    : Copy CciChar strings, like strncpy                                      */
/*   CciCharCmp     : Compare CciChar strings, like strcmp                                    */
/*   CciCharLen     : Find length of CciString, like strlen, result in characters (not bytes) */
/*   CciString      : Convert an ordinary string to Unicode, result must be free()'d          */
/*                    after use, intended for use with source-code constants                  */
/*   CciNString     : Convert N bytes of a string to Unicode, result must be free()'d         */
/*                    after use, intended for use with source-code constants                  */
/*   mbString       : Convert Unicode to an ordinary string, result must be free()'d          */
/*                    after use, intended for use to produce human-readable text              */
/*                                                                                            */
/**********************************************************************************************/
int CciCharIsSpace(const CciChar c)
{
  if (((c >= chTab) && (c <= chCR)) || (c == chSpace))
  {
     return 1;
  }
  else
  {
     return 0;
  }
}

int CciCharIsAlnum(const CciChar c)
{
  if (((c >= chDigit_0) && (c <= chDigit_9)) ||
      ((c >= chLatin_A) && (c <= chLatin_Z)) ||
      ((c >= chLatin_a) && (c <= chLatin_z)))
  {
     return 1;
  }
  else
  {
     return 0;
  }
}

void CciCharCpy(CciChar* target, const CciChar* source)
{
  CciChar c ;
  for (;;)
    {
      c = *source ;
      source += 1 ;
      *target = c ;
      target += 1 ;
      if ( 0 == c ) break ;
    }
}

void CciCharNCpy(CciChar* target, const CciChar* source, unsigned int size)
{
  CciChar c ;
  int i;
  for (i=0;i<size;i+=1)
    {
      c = *source ;
      *target = c ;
      target += 1 ;
      if ( 0 != c ) source += 1 ;
    }
}

int CciCharLen(const CciChar* s)
{
  int i=0 ;
  while (*s != 0 )
    {
      s += 1 ;
      i += 1 ;
    }
  return i ;
}

int CciCharCmp(const CciChar * s1, const CciChar* s2)
{
  CciChar c1 ;
  CciChar c2 ;
  for (;;)
    {
      c1 = *s1 ;
      s1 += 1 ;
      c2 = *s2 ;
      s2 += 1 ;
      if ( (0 == c1) || (c1 != c2) ) break ;
    }
  return c1-c2 ;
}

const CciChar* CciString(const char* source)
{
  /* Maximum number of characters in Unicode representation */
  int maxChars = strlen(source) + 1 ;
  CciChar* buffer = (CciChar*)malloc(maxChars * sizeof(CciChar)) ;
  int rc ;
  /* We don't know the compiler's code page at this stage, but 1208 (utf8 Unicode) */
  /* will generally be appropriate for ASCII systems                               */
  cciMbsToUcs(&rc, source, buffer, maxChars, 1208) ;
  return buffer ;
}

const CciChar* CciNString(const char* source, const unsigned int size)
{
  /* Maximum number of characters in Unicode representation */
  /* The bytes may not be NULL terminated so we need to make a copy */
  unsigned int bufferSize = ((size + 1) * sizeof(CciChar));
  int rc;
  CciChar* buffer = (CciChar*)malloc(bufferSize);
  memset(buffer, '\0', bufferSize);
  memcpy(buffer, source, size);
  /* We don't know the compiler's code page at this stage, but 1208 (utf8 Unicode) */
  /* will generally be appropriate for ASCII systems                               */
  cciMbsToUcs(&rc, source, buffer, size, 1208) ;
  return buffer ;
}

const char* mbString(const CciChar* source)
{
  int charCount = CciCharLen(source) ;
  int maxBytes = (charCount+1) * 9 ;
  char * buffer = (char *)malloc(maxBytes) ;
  int rc ;
  cciUcsToMbs(&rc, source, buffer, maxBytes, 1208) ;
  return buffer ;
}

