/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Thomas Sallantin                Date: April 2001                   */
/*                                                                            */
/******************************************************************************/
/* Version 1.0   First release NT Check Duplication                           */
/******************************************************************************/
/*                                                                            */
/* Note: This plugin node implementation is not necessarily portable          */
/* to all platforms supported by MQSeries Integrator Version 2.               */
/*                                                                            */
/******************************************************************************/

#include <winbase.h>

#ifndef H_CheckDuplication
#define H_CheckDuplication

#include <BipCni.h>
#include <BipCci.h>

#if defined(__CCI_SOLARIS__) 
#include <BipSampPluginUtil.h>
#endif

#ifndef max                                   
#define max(x, y) (((x) > (y)) ? (x) : (y))   
#endif                                        
                                              
#ifndef min                                   
#define min(x,y) (((x)<(y)) ? (x) : (y))      
#endif                                        

#if defined(__CCI_SOLARIS__) 
/* ---------------------------------------------------------------- */
/*                     Internal constants                           */
/*                                                                  */
/* The interface expects some CciChar strings to be passed. These   */
/* have to be constructed from char strings using the supplied      */
/* utilities. The char constants are defined here along with the    */
/* static CciChar variables which will contain these constants      */
/* converted into CciChar format.                                   */
/*                                                                  */
/* ---------------------------------------------------------------- */
//#define CONST_ZERO                           "0"
//const CciChar * constZero                   = 0;

#define CONST_MQSERIES_INTEGRATOR_2          "MQSIv201"
const CciChar * constMQSeriesIntegrator2     = 0;

#define CONST_PLUGIN_NODE                    "CheckDuplicationNode"
const CciChar * constPlugInNode              = 0;

#define CONST_PLUGIN_NODE_FACTORY            "CheckDuplicationNodeFactory"
const CciChar * constPluginNodeFactory       = 0;

#define CONST_OUT                            "out"
const CciChar * constOut                     = 0;

#define CONST_FAILURE                        "failure"
const CciChar * constFailure                = 0;
#endif

#define MAX_ATTR_NAME_LEN     128
#define MAX_NODE_NAME_LEN     128
#define MAX_STRING_SIZE       256
#define MAX_TERMINAL_NAME_LEN 256
#define CNI_TYPE_INTEGER        1
#define CNI_TYPE_STRING         2

#define CORRECTNO             0
#define CORRECTYES            1

#define MQMDIDNO			  0
#define MQMDIDYES			  1

#define MAX_ATTRIBUTE_NAME_LEN     128
#define MAX_NODE_NAME_LEN          128
#define MAX_TERMINAL_NAME_LEN      256

#if defined(__CCI_AIX__)
   /* http://www.erlenstar.demon.co.uk/unix/faq_2.html */
   extern char **environ;
   #define _environ environ
#endif

#if defined(__CCI_SOLARIS__)
   extern char **environ;        /* not tested */
   #define _environ environ      /* not tested */
#endif

#if defined(__CCI_SOLARIS__)
#define CCI_CHECK_RC()                                                                        \
   if (rc != CCI_SUCCESS) {                                                                   \
     if (rc == CCI_EXCEPTION) cciRethrowLastException(0);                                     \
     else {                                                                                   \
       cciLog(&rc, CCI_LOG_ERROR, __FILE__, __LINE__, "",                                     \
              (CciChar*)constMQSeriesIntegrator2,                                             \
              0, "", 0);                                                                      \
       return;                                                                                \
     }                                                                                        \
   }
#else
#define CCI_CHECK_RC()                                                                        \
   if (rc != CCI_SUCCESS) {                                                                   \
     if (rc == CCI_EXCEPTION) cciRethrowLastException(0);                                     \
     else {                                                                                   \
       cciLog(&rc, CCI_LOG_ERROR, __FILE__, __LINE__, "", L"MQSIv201", 0, "", 0);             \
       return;                                                                                \
     }                                                                                        \
   }
#endif


typedef struct attributeTblEntry {
   void*    next;                                     /* Pointer to next entry */
   void*    valptr;                                   /* Pointer to value */
   int      type;                                     /* Value type */
   CciChar  name[MAX_ATTR_NAME_LEN];                  /* Attribute name */
} ATTRIBUTE_TBL_ENTRY;


typedef struct terminalListEntry {
   void*        next;                                 /* Pointer to next entry */
   CciTerminal* handle;                               /* Handle to terminal object */
   CciChar      name[MAX_TERMINAL_NAME_LEN];          /* Terminal name */
} TERMINAL_LIST_ENTRY;


typedef struct context {
   CciNode*             nodeObject;                   /* Handle for node object */
   ATTRIBUTE_TBL_ENTRY* attrListHead;                 /* Head of linked list of attributes */
   ATTRIBUTE_TBL_ENTRY* attrListPrevious;             /* Previous entry in attribute list */
   TERMINAL_LIST_ENTRY* inputTerminalListHead;        /* Head of linked list of input terminal handles */
   TERMINAL_LIST_ENTRY* inputTerminalListPrevious;    /* Previous entry in input terminal list */
   TERMINAL_LIST_ENTRY* outputTerminalListHead;       /* Head of linked list of output terminal handles */
   TERMINAL_LIST_ENTRY* outputTerminalListPrevious;   /* Previous entry in output terminal list */
   CciChar              nodeName[MAX_NODE_NAME_LEN];  /* Node name */
} NODE_CONTEXT_ST;



/* Function Prototypes */

/* Node Utility Functions */
ATTRIBUTE_TBL_ENTRY *insertAttributeTableEntry
(
   NODE_CONTEXT_ST *context,
   wchar_t    *nodeName, 
   wchar_t    *attributeName,
   int         type
);


void *getInputTerminalHandle(
   NODE_CONTEXT_ST* context,
   CciChar*         terminalName
);


TERMINAL_LIST_ENTRY *insertInputTerminalListEntry
(
   NODE_CONTEXT_ST *context,
   wchar_t *terminalName 
);

void *getOutputTerminalHandle(
   NODE_CONTEXT_ST* context,
   CciChar*         terminalName
);


TERMINAL_LIST_ENTRY *insertOutputTerminalListEntry
(
   NODE_CONTEXT_ST *context,
   wchar_t *terminalName 
);



/* Node Implementation Functions */
void *createNodeContext( void    *factoryObject,
                         wchar_t *nodeName,
                         void    *nodeObject );

void deleteNodeContext(CciContext* context);

int getAttributeName( CciContext* context,
                      int         index,
                      CciChar*    buffer,
                      int         bufsize );

int setAttribute( CciContext* context,
                  CciChar*    attrName,
                  CciChar*    attrValue );

int getAttribute( CciContext* context,
                  CciChar*    attrName,
                  CciChar*    buffer,
                  int         bufsize );

char* UnicodeToASCII(const wchar_t* lStr);
wchar_t* ASCIIToUnicode(const char* aStr);
wchar_t* ASCIInToUnicode(const char* aStr, size_t len);

void wgetNameValue(char *Env, CciChar **Name, CciChar **Value);

CciChar *wStrUpper(CciChar *string);
CciChar *wStrLower(CciChar *string);


CciMessage *CheckDuplicationMsgError(CciMessage *message, CciChar *msg,CciChar *codemsg);

#endif